@@ -467,11 +467,7 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	  "(%d -> %d)",entry->itf->number,itf->number);
     if (!entry->addr) entry->addr = alloc(sizeof(*addr));
     *entry->addr = *addr;
-    /* need to revise this policy if we get user-settable QOS @@@ */
-    memset(&entry->qos,0,sizeof(entry->qos));
-    entry->qos.txtp.traffic_class = ATM_UBR;
-    entry->qos.txtp.max_sdu = RFC1483LLC_LEN+RFC1626_MTU;
-    entry->qos.rxtp = entry->qos.txtp;
+    entry->qos = entry->itf->qos;
     for (walk = entry->vccs; walk; walk = walk->next)
 	if (!walk->connecting)
 	    if (set_ip(walk->fd,ip) < 0)
@@ -799,6 +795,12 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
     }
     switch (cmd) {
 	case SIOCSARP:
+	    if (req->arp_flags & ATF_DEFQOS) {
+		diag(COMPONENT,DIAG_DEBUG,"got SIOCSARP ATF_DEFQOS for itf %d",
+		  itf_num);
+		itf->qos = req->arp_qos;
+		return 0;
+	    }
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,
 	      (struct sockaddr *) &req->arp_ha,pretty) < 0) {
 		diag(COMPONENT,DIAG_ERROR,"a2t fails on SIOCSARP");
@@ -807,6 +809,7 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
 	    diag(COMPONENT,DIAG_DEBUG,"got SIOCSARP for itf %d, IP %d.%d.%d.%d"
 	      ", ATM %s, flags 0x%x",itf_num,ipp[0],ipp[1],ipp[2],ipp[3],buffer,
 	      req->arp_flags);
+	    if (!req->arp_qos.txtp.traffic_class) req->arp_qos = itf->qos;
 	    switch (req->arp_ha.sas_family) {
 		case AF_ATMPVC:	
 		    adjust_qos(itf,&req->arp_qos,req->arp_flags & ATF_NULL);
