@@ -9,6 +9,8 @@
 #include <linux/atmdev.h>
 #include <linux/sonet.h>
 
+#include "/usr/src/linux/drivers/atm/eni.h"
+
 
 static void usage(const char *name)
 {
@@ -28,8 +30,8 @@ int main(int argc,char **argv)
 	return 1;
     }
     req.number = atoi(argv[1]);
-    if (ioctl(s,12345678,&req) < 0) {
-	perror("ioctl 12345678");
+    if (ioctl(s,ENI_MEMDUMP,&req) < 0) {
+	perror("ioctl ENI_MEMDUMP");
 	return 1;
     }
     return 0;
