@@ -43,6 +43,7 @@
 #include <linux/atmclip.h>
 #include "io.h"
 #include "atmd.h"
+#include "atm.h"
 
 #define SNMP_VCI 16
 #define COMPONENT "IO"
@@ -91,6 +92,7 @@ void update_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
 {
   struct atmif_sioc req;
   struct sockaddr_atmsvc addr, ouraddr[MAX_EXTRA_ADDRS+1];
+  char buffer[MAX_ATM_ADDR_LEN+1];
   int fd, m, n;
 
   addr.sas_family = AF_ATMSVC;
@@ -124,30 +126,47 @@ void update_nsap(int itf, AsnOid *netprefix, AsnOid *esi)
   if (ioctl(fd, ATM_GETADDR, &req) <0)
     diag(COMPONENT, DIAG_FATAL, "ioctl ATM_GETADDR: %s", strerror(errno));
 
-  n = 1;
+  n = 0;
+  if (req.length && atm_equal(&addr, &ouraddr[0], ATM_ESA_LEN, 0)) {
+    diag(COMPONENT, DIAG_INFO, "Primary ATM Address did not change");
+    n = 1;
+  }
 
-  for(m = 0; m < netprefix->octetLen; m++)
-     if(addr.sas_addr.prv[m] != ouraddr[0].sas_addr.prv[m])
-     {	n = 0;
-	break;
-     }
-
-  if ((req.length == 0) || (n == 0)) {
+  if ((!(m = req.length)) || (!n)) {
   	  req.number = itf;
   	  req.arg = NULL;
   	  req.length = 0;
 
-  	  if(ioctl(fd, ATM_RSTADDR, &req) < 0)
+  	  if (ioctl(fd, ATM_RSTADDR, &req) < 0)
     	    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_RSTADDR: %s", strerror(errno));
 
 	  req.number = itf;
 	  req.arg = &addr;
-	  req.length = sizeof(addr);
-	  
-	  if(ioctl(fd, ATM_ADDADDR, &req) < 0)
+	  req.length = sizeof(addr); 
+
+	  if (ioctl(fd, ATM_ADDADDR, &req) < 0)
 	    diag(COMPONENT, DIAG_FATAL, "ioctl ATM_ADDADDR: %s", strerror(errno));
 
-	  diag(COMPONENT, DIAG_INFO, "ATM NSAP Address set on local host");
+	  atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr, A2T_PRETTY);
+
+	  diag(COMPONENT, DIAG_INFO, "Primary ATM Address %s added local", buffer);
+	
+	  for (n = 0; n < m/sizeof(addr); n++) {
+
+		  if(n > MAX_EXTRA_ADDRS-1) break; /* We have already registered "primary" NSAP */
+
+		  req.number = itf;
+		  req.arg = &ouraddr[n];
+		  req.length = sizeof(*ouraddr); 
+	  
+		  atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &ouraddr[n], A2T_PRETTY);
+
+		  if (ioctl(fd, ATM_ADDADDR, &req) < 0)
+		    diag(COMPONENT, DIAG_ERROR, "ioctl ATM_ADDADDR: %s", strerror(errno));
+	          else	
+		    diag(COMPONENT, DIAG_INFO, "Extra   ATM Address %s added local", buffer);
+
+	  }
   }	     	
   close(fd);
 
