@@ -92,12 +92,12 @@ static void construct(FIELD *start)
 }
 
 
-static void parser(FIELD *start,int level)
+static void parser(FIELD *start,int level,int group)
 {
     FIELD *walk;
     TAG *scan;
     VALUE_LIST *tag;
-    int count;
+    int count,patch_here;
     BREAK *brk,*tmp_brks,*next;
 
     for (walk = start; walk; walk = walk->next) {
@@ -159,7 +159,9 @@ static void parser(FIELD *start,int level)
 			patch(scan->patch,pc-scan->patch-1);
 			for (tag = scan->more; tag; tag = tag->next)
 			    patch(tag->patch,pc-tag->patch-1);
-			parser(scan->block,level+1);
+			if (!scan->block && scan->abort_id)
+			    code("%s%s\n","OP_ABORT",scan->abort_id);
+			parser(scan->block,level+1,scan->group);
 			if (scan->next) {
 			    code("%s%s\n","OP_JUMP","?");
 			    scan->patch = pc-1;
@@ -176,8 +178,19 @@ static void parser(FIELD *start,int level)
 		case vt_length:
 		    code("%s%d%d%d/* %s */\n","OP_BEGIN_LEN",walk->jump,
                       walk->pos,walk->size,walk->id);
+		    if (walk->value->recovery) {
+			code("%s%s%d%d\n","OP_BEGIN_REC",walk->value->recovery,
+			    group,"?");
+			patch_here = pc-1;
+		    }
 		    tmp_brks = brks;
-		    parser(walk->value->block,level+1);
+		    if (!walk->value->block && walk->value->abort_id) 
+			code("%s%s\n","OP_ABORT",walk->value->abort_id);
+		    parser(walk->value->block,level+1,group);
+		    if (walk->value->recovery) {
+			code("%s\n","OP_END_REC");
+			patch(patch_here,pc);
+		    }
 		    for (brk = brks; brk; brk = next) {
 			next = brk->next;
 			patch(brk->pc,pc-brk->pc-1);
@@ -218,7 +231,7 @@ void third(FIELD *def)
 	to_dump("static DUMP_FIELD dump_fields[] = {\n");
     }
     begin_code();
-    parser(def,0);
+    parser(def,0,0);
     parser_size = end_code()+1;
     to_c("    OP_END\n};\n\n");
     if (dump) to_dump("};\n\n");
